#' Residential Population Down-Scaling
#'
#' @param .target an object of class \code{sf} representing individual
#'     building polygons
#' @param source an object of class \code{sf} representing coarse polygon
#'     features (such as census tracts or city blocks) including
#'     population counts
#' @param sid source id
#' @param spop source population
#' @param volume target volume (height or number of floors)
#' @param point whether to use point geometries (TRUE/FALSE)
#' @param method available methods: awi, vwi
#'
#' @return and object of class \code{sf} including estimated population counts
#'     for target zone features using either awi or vwi methods
#' @export
#'
#' @importFrom rlang quo_name
#' @importFrom rlang enquo
#' @importFrom sf st_crs
#'
#' @examples
#' # read lib data
#' data('source')
#' data('target')
#'
#' # areal weighted interpolation - awi
#' pp_estimate(target, source = source, sid = sid, spop = pop,
#'     method = awi)
#'
#' # areal weighted interpolation - awi using point geometries
#' pp_estimate(target, source = source, sid = sid, spop = pop,
#'     method = awi, point = TRUE)
#'
#' # volume weighted interpolation - vwi
#' pp_estimate(target, source = source, sid = sid, spop = pop,
#'     method = vwi, volume = floors)
#'
#' # volume weighted interpolation - vwi using point geometries
#' pp_estimate(target, source = source, sid = sid, spop = pop,
#'     method = vwi, volume = floors, point = TRUE)
#'
pp_estimate <- function(.target, source, sid, spop, volume = NULL, point = FALSE, method) {
  # check arguments
  if (missing(.target)) {
    stop('target is required')
  }

  if (missing(source)) {
    stop('source is required')
  }

  if (missing(sid)) {
    stop('source id is required')
  }

  if (missing(spop)) {
    stop('source population is required')
  }

  if (missing(method)) {
    stop('method is required')
  }

  # enquote args where necessary
  sid <- rlang::quo_name(rlang::enquo(sid))
  spop <- rlang::quo_name(rlang::enquo(spop))
  volume <- rlang::quo_name(rlang::enquo(volume))
  method <- rlang::quo_name(rlang::enquo(method))

  # check whether source and .target are of sf class
  tc <- 'sf' %in% class(.target)
  sc <- 'sf' %in% class(source)

  if (tc != sc) {
    stop('source and target must be objects of sf class')
  }

  # check whether source and target share the same crs
  t_crs <- sf::st_crs(.target)
  s_crs <- sf::st_crs(source)

  if (t_crs != s_crs) {
    stop('CRS mismatch')
  }

  # check whether params exist in the given source object
  if (!sid %in% colnames(source)) {
    stop(sprintf('%s cannot be found in the given source object', sid))
  }

  if (!spop %in% colnames(source)) {
    stop(sprintf('%s cannot be found in the given source object', spop))
  }

  # check whether method is valid
  m <- c('awi', 'vwi')

  if (!method %in% m) {
    stop('please provide avalid method: awi, vwi')
  }

  # check whether spop is numeric
  if (!is.numeric(source[, spop, drop = TRUE])) {
    stop('source population must be numeric')
  }

  # check whether point is logical
  if (!is.logical(point)) {
    stop('point must be logical')
  }

  # check whether sid and pop already exists in both target and source features
  if (any(colnames(.target) == sid)) {
    colnames(.target)[colnames(.target) == sid] <- 'target_id'
  }

  if (any(colnames(.target) == spop)) {
    colnames(.target)[colnames(.target) == spop] <- 'target_pop'
  }

  if (method == 'awi') {
    out <- pp_awi(.target, source = source, sid = sid, spop = spop,
                  point = point)
  } else if (method == 'vwi') {
    if (volume == 'NULL') {
      stop('volume is required for vwi')
    }

    if (!volume %in% colnames(.target)) {
      stop(sprintf('%s cannot be found in the given target object', volume))
    }

    if (!is.numeric(.target[, volume, drop = TRUE])) {
      stop('volume must be numeric')
    }

    out <- pp_vwi(.target, source = source, sid = sid, spop = spop,
                  volume = volume, point = point)
  }

  return(out)
}
